#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# +------------------------------------------------------------+
# |                                                            |
# |             | |             | |            | |             |
# |          ___| |__   ___  ___| | ___ __ ___ | | __          |
# |         / __| '_ \ / _ \/ __| |/ / '_ ` _ \| |/ /          |
# |        | (__| | | |  __/ (__|   <| | | | | |   <           |
# |         \___|_| |_|\___|\___|_|\_\_| |_| |_|_|\_\          |
# |                                   custom code by SVA       |
# |                                                            |
# +------------------------------------------------------------+
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   Copyright (C) 2023  SVA System Vertrieb Alexander GmbH
#                       by gerd.stolz@sva.de

from typing import Any, Dict, Generator, Mapping, Optional, Union
from cmk.base.api.agent_based.checking_classes import IgnoreResults, IgnoreResultsError
from cmk.base.api.agent_based.type_defs import Parameters, StringTable
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    render,
    Result,
    Metric,
    State,
    Service,
    ServiceLabel,
    check_levels,
    get_value_store,
)


def parse_m365_service_health(string_table: StringTable) -> Any:
    import json

    parsed = {}
    for item in json.loads("".join(string_table[0])):
        parsed[item["service_name"]] = item
    return parsed


register.agent_section(
    name="m365_service_health",
    parse_function=parse_m365_service_health,
)

state_map = {
    "serviceOperational": State.OK,
    "investigating": State.WARN,
    "restoringService": State.WARN,
    "verifyingService": State.WARN,
    "serviceRestored": State.OK,
    "postIncidentReviewPublished": State.OK,
    "serviceDegradation": State.CRIT,
    "serviceInterruption": State.CRIT,
    "extendedRecovery": State.WARN,
    "falsePositive": State.OK,
    "investigationSuspended": State.WARN,
    "resolved": State.OK,
    "mitigatedExternal": State.WARN,
    "mitigated": State.WARN,
    "resolvedExternal": State.OK,
    "confirmed": State.WARN,
    "reported": State.WARN,
    "unknownFutureValue": State.WARN,
}


def discover_m365_service_health(section: Any) -> Generator[Service, None, None]:
    """
    params: Parameters etwa register-discovery_default_parameters,
            sowie WATO-Regeln
    """
    for k in section.keys():
        yield Service(item=k)


def check_m365_service_health(
    item: str, section: Any
) -> Generator[Union[Result, Metric], None, None]:
    """
    item: Name des M365 service der gecheckt werden soll
    section: Daten wie aus parse-funcntion
    """

    # Beispiel: Ermittlung Werte für item (im Fall multiple Service-Items!)
    values = section.get(item, None)
    if values == None:
        return
    details = ""
    for key, value in values.items():
        if key.startswith("last_issue"):
            key = key.replace("_", " ")
            details += f"{key}: {value}\n"
    # Beispiel: Result + Metrics manuell yielden. Für jede Einzelne Wert/Status
    #           ein "Result" und ggf Metric yielden! CheckMK ermittelt aus
    #           allen TeilResults dann Gesamt-Result des Service
    yield Result(
        state=state_map.get(values["service_status"], State.UNKNOWN), 
        summary=f"Service is in state: {values['service_status']}",                                       
        details=f"{details}",  # 4)
    )
    # 1) State.OK, WARN, CRIT (UNKNOWN)
    # 2) summary wird in Serviceübersciht angezeigt. Keine Punkte/Komma am
    #    Text-Ende
    # 3) notice, wie summary, wird aber nur bei Non-OK gezeigt!
    # 4) details, erweiterter Text, z.B. Einzelwerte die zum Gesamtwert führten
    #    default ist summary oder notice


register.check_plugin(
    name="m365_service_health",
    service_name="Microsoft %s",  # %s = Platzhalten Item-Name!
    check_function=check_m365_service_health,
    discovery_function=discover_m365_service_health,
    # check_ruleset_name="check_m365_service_health" # check ist nicht konfigurierbar
)
